#include <windows.h>
#include <stdio.h>

#define NUM_JOYS 4

void send_rumble_cmd( HANDLE handle, int joynum, unsigned addr_hi, unsigned addr_lo, unsigned data);

//
// This is a sample application which demonstrates the Adaptoid programming API.
//
// This program will poll up to 4 controllers, reporting joystick positions,
// button states, and controller pak status.  If the "A" button on any controller
// is pressed, the rumble pack on that controller will rumble.
//
// Press any key on the keyboard to quit.
//

_cdecl main(int argc, char **argv)
{
    unsigned char buf[64], *p;
    unsigned char pak[64], *k;
    int i,j;
    unsigned char joydata[4][4];
    HANDLE handle;
    int nbytes;
    char nAdaptoids;

    handle = CreateFile( "\\\\.\\Wish_NA1",  // the name of the Adaptoid driver
                         GENERIC_READ | GENERIC_WRITE,
                         FILE_SHARE_READ | FILE_SHARE_WRITE,
                         NULL, OPEN_EXISTING, 0, NULL);

    if (handle == INVALID_HANDLE_VALUE) 
    {
        printf("Could not find Adaptoid\n");
        exit(0);
    }

    //
    // Ask the driver for the number of Adaptoids plugged in. Anytime you
    // read 1 byte from the driver, it will return the number of Adaptoids
    // as an integer.
    //
    ReadFile( handle, &nAdaptoids, 1, &nbytes, NULL);
    printf("There are %d Adaptoids plugged in\n", nAdaptoids);
    printf("Press the 'A' button on the controller to rumble\n");

    printf("Press any key on the keyboard to quit\n");

    memset(joydata, 0, sizeof(joydata));

    while (!_kbhit())
    {
        //
        // Build a command block to read NUM_JOYS joysticks
        //
        p = buf;
        for (i=0; i < NUM_JOYS; i++)
        {
            *p++ = 1;   // # of bytes to send - just the command byte
            *p++ = 4;   // # of bytes to get
            *p++ = 1;   // command to send (1 = read joystick)

            // Initialize space for the returned data
            for (j=0; j < 4; j++)
                *p++ = 0xff;
        }
        *p++ = 0xfe;  // end of command block

        buf[63] = 1;  // set last byte to 1 to process command block

        // Send the command block and get the results. Must be 64 bytes.
        WriteFile( handle, buf, 64, &nbytes, NULL);
        ReadFile( handle, buf, 64, &nbytes, NULL);
        
        //
        // Build a command block to check for memory paks or rumble paks
        //
        k = pak;
        for (i=0; i < NUM_JOYS; i++)
        {
            *k++ = 1;   // # of bytes to send - just the command byte
            *k++ = 3;   // # of bytes to get
            *k++ = 0;   // command to send (0 = check for mempak and enable mempak I/O)

            // Initialize space for the returned data
            for (j=0; j < 3; j++)
                *k++ = 0xff;
        }
        *k++ = 0xfe;  // end of command block

        pak[63] = 1;  // set last byte to 1 to process command block

        // Send the command block and get the results. Must be 64 bytes.
        WriteFile( handle, pak, 64, &nbytes, NULL);
        ReadFile( handle, pak, 64, &nbytes, NULL);
        
        // process results
        p = buf;
        k = pak;
        for (i=0; i < NUM_JOYS; i++)
        {
            int skip = 0;

            p++;              // skip over send count
            k++;

            if (*p++ & 0xC0)
                skip = 1;     // this joystick isn't talking
            k++;           

            p++;              // skip over command byte
            k++;

            if (!skip && memcmp(joydata[i], p, 4) != 0)
            {
                // joystick data changed
                memcpy(joydata[i], p, 4);

                // print out joystick data
                printf("Controller #%d:  buttons=%02x%02x, X=%d, Y=%d, %s pak\n",
                       1+i, joydata[i][0], joydata[i][1], (char)joydata[i][2], (char)joydata[i][3], 
                       (k[2] & 1) ? "has" : "no");

                if (k[2] & 1)
                {
                    // enable rumble pack
                    send_rumble_cmd( handle, i, 0x80, 0x01, 0x80);

                    // turn rumble motor on or off depending on "A" button
                    if (joydata[i][0] & 0x80)              
                        send_rumble_cmd( handle, i, 0xC0, 0x1B, 0x01);  // on 
                    else
                        send_rumble_cmd( handle, i, 0xC0, 0x1B, 0x00);  // off
                }
            }

            p += 4;  // skip over joystick data
            k += 3;  // skip over pak status
        }
    }

    CloseHandle( handle);
}


void send_rumble_cmd( HANDLE handle, int joynum, unsigned addr_hi, unsigned addr_lo, unsigned data)
{
    unsigned char rumble[64], *r;
    int j, k;
    int nbytes;

    // select which joystick to send rumble command to
    r = rumble;
    for (j=0; j < joynum; j++)
        *r++ = 0;

    *r++ = 35;       // # of bytes to send 
    *r++ = 1;        // # of bytes to get
    *r++ = 3;        // command to send (3 = write controller pak)
    *r++ = addr_hi;  // controller pak address
    *r++ = addr_lo;  // controller pak address

    for (k=0; k < 32; k++)
        *r++ = data;        // controller pak data

    *r++ = 0xff;            // initialize space for return data
    *r++ = 0xfe;            // end of command block

    rumble[63] = 1;  // set last byte to 1 to process command block

    // Send the command block and get the results. Must be 64 bytes.
    WriteFile( handle, rumble, 64, &nbytes, NULL);
    ReadFile( handle, rumble, 64, &nbytes, NULL);
}
